library crud;

import 'package:flutter/foundation.dart';
import 'package:restified/http.dart';
import 'package:rxdart/rxdart.dart';

class GetBloc<T> {
  final String endpoint;
  final Serializable<T> model;

  final BehaviorSubject<List<T>> _all = BehaviorSubject<List<T>>();
  final BehaviorSubject<T> _single = BehaviorSubject<T>();

  GetBloc({this.endpoint, this.model});

  getAll() async {
    _all.sink.add(null);
    List<T> response = await Http().getAll('$endpoint', model);
    _all.sink.add(response);
  }

  getById(int id) async {
    _single.sink.add(null);
    T m = await Http().getSingle('$endpoint/$id', model);
    _single.sink.add(m);
  }

  change(T model) {
    _single.sink.add(model);
  }

  dispose() {
    _all.close();
    _single.close();
  }

  BehaviorSubject<List<T>> get all => _all;

  BehaviorSubject<T> get single => _single;
}

class ManipulationBloc {
  final String endpoint;

  final BehaviorSubject<UpdateResponse> _updateResponse =
      BehaviorSubject<UpdateResponse>();
  final BehaviorSubject<InsertResponse> _insertResponse =
      BehaviorSubject<InsertResponse>();
  final BehaviorSubject<bool> _loading = BehaviorSubject<bool>();

  ManipulationBloc({@required this.endpoint}) {
    _loading.sink.add(false);
  }

  dispose() {
    _updateResponse.close();
    _insertResponse.close();
    _loading.close();
  }

  update(int id, Map model) async {
    _loading.sink.add(true);
    UpdateResponse resp = await Http().put('$endpoint/$id', model);
    _updateResponse.sink.add(resp);
    _loading.sink.add(false);
  }

  insert(Map model) async {
    _loading.sink.add(true);
    InsertResponse resp = await Http().post('$endpoint', model);
    _insertResponse.sink.add(resp);
    _loading.sink.add(false);
  }

  BehaviorSubject<UpdateResponse> get updateResponse => _updateResponse;

  BehaviorSubject<InsertResponse> get insertResponse => _insertResponse;

  BehaviorSubject<bool> get loading => _loading;
}
