library http;

import 'dart:convert';

import 'package:http/http.dart' as http;
import 'package:shared_preferences/shared_preferences.dart';

abstract class Serializable<T> {
  T fromJson(Map json);
  Map<String, dynamic> toJson();
  T clone();
}

class Response {
  final bool success;
  final String msg;

  const Response({this.success, this.msg});

  factory Response.fromJson(Map json) {
    return Response(
      success: json['success'],
      msg: json['msg'],
    );
  }
}

class UpdateResponse {
  Response response;
  int affectedRows;

  UpdateResponse({this.affectedRows});

  UpdateResponse.fromJson(Map json)
      : response = Response.fromJson(json),
        affectedRows =
            json['data'] != null ? json['data']['affected_rows'] : null;
}

class InsertResponse {
  Response response;
  int id;

  InsertResponse({this.id});

  InsertResponse.fromJson(Map json)
      : response = Response.fromJson(json),
        id = json['data'] != null ? json['data']['id'] : null;
}

class Http {
  static String api;

  Future<T> getSingle<T>(String endpoint, Serializable<T> s) async {
    http.Response response = await http
        .get('$api$endpoint', headers: {'Authorization': await _getToken()});
    return s.fromJson(json.decode(response.body)['data']);
  }

  Future<List<T>> getAll<T>(String endpoint, Serializable<T> s) async {
    http.Response response = await http
        .get('$api$endpoint', headers: {'Authorization': await _getToken()});
    dynamic j = json.decode(response.body)['data'];
    return (j as List).map((i) => s.fromJson(i)).toList();
  }

  Future<InsertResponse> post(String endpoint, Map s) async {
    http.Response response = await http.post('$api$endpoint',
        headers: {
          'Authorization': await _getToken(),
          'Content-Type': 'application/json'
        },
        body: json.encode(s));
    return InsertResponse.fromJson(json.decode(response.body));
  }

  Future<UpdateResponse> put(String endpoint, Map s) async {
    http.Response response = await http.put('$api$endpoint',
        headers: {
          'Authorization': await _getToken(),
          'Content-Type': 'application/json'
        },
        body: json.encode(s));
    return UpdateResponse.fromJson(json.decode(response.body));
  }

  Future<Response> delete(String endpoint) async {
    http.Response response = await http.delete('$api$endpoint', headers: {
      'Authorization': await _getToken(),
    });
    return Response.fromJson(json.decode(response.body));
  }

  Future<String> _getToken() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    return prefs.getString('token');
  }
}
